## @knitr knitrSetup
knit_hooks$set(error = function(x, options) stop(x)) 


## @knitr libRequirements
options(stringsAsFactors=FALSE)

require(GenomicRanges)
require(rtracklayer)
require(Biostrings)
require(Biobase)
require(xtable)
require(ggplot2)
require(Cairo)


## @knitr getUTRs
rn5_utrLoc <- GRanges(seqnames=Rle("chr18"), ranges=IRanges(start=25680515, end=25690671), strand="+")
rn5_utrSeq <- read.DNAStringSet("rn5_camk4utr.fa", use.names=T)

hg19_utrLoc <- GRanges(seqnames=Rle("chr5"), ranges=IRanges(start=110820197, end=110830455), strand="+") # on chr5
hg19_utrSeq <- read.DNAStringSet("hg19_camk4utr_rn5Overlap.fa", use.names=T)

mm10_utrLoc <- GRanges(seqnames=(Rle("chr18")), ranges=IRanges(start=33185204, end=33195767), strand="+")
mm10_utrSeq <- read.DNAStringSet("mm10_camk4utr_rn5Overlap.fa", use.names=T)


## @knitr utrLocTable
utrTable <- data.frame(Organism=c("R. norvegicus", "M. musculus", "H. sapiens"), Chromosome=c("Chr 18", "Chr 18", "Chr 5"), Start=c(start(rn5_utrLoc), start(mm10_utrLoc), start(hg19_utrLoc)), End=c(end(rn5_utrLoc), end(mm10_utrLoc), end(hg19_utrLoc)))

print(xtable(utrTable), type = 'html', html.table.attributes = 'style="border-spacing:20px 5px;"', include.rownames=FALSE)


## @knitr getPWM
pwmFiles <- dir("PWMDir", pattern=".pwm")
pwmFiles <- file.path("PWMDir", pwmFiles)

pwmDat <- lapply(pwmFiles, function(x){
	pwmTmp <- read.table(x, header=F, stringsAsFactors=F)
	row.names(pwmTmp) <- c("A","C","G","T")
	pwmTmp <- as.matrix(pwmTmp)
	pwmTmp
})

pwmNames <- basename(pwmFiles)
pwmNames <- substr(pwmNames, 1, nchar(pwmNames)-4)
names(pwmDat) <- pwmNames

pwmID <- read.table("PWMDir/matrix_list.txt", sep="\t", header=F)
protID <- sapply(strsplit(pwmID[,2], "_"), function(x){x[1]})

allData <- scan("PWMDir/RBPDB_v1.3_proteins_2012-09-28.tdt.txt", what=character(), sep="\n" )
pwmSplit <- strsplit(allData, "\t")
tmpDat <- matrix(pwmSplit[[1]], nrow=1, ncol=15)
invisible(lapply(pwmSplit[2:length(pwmSplit)], function(x){
	tmpDat <<- rbind(tmpDat, matrix(x, nrow=1, ncol=15))
}))

pwmInfo <- as.data.frame(tmpDat)
names(pwmInfo) <- c("protID", "databaseID", "date1", "date2", "geneSymbol", "geneName",
										"species", "specID", "bindingTypes", "flag1", "comment", "otherID",
										"otherID2", "otherID3", "otherID3")
rownames(pwmInfo) <- pwmInfo$protID


## @knitr findPWMs
# inSeq: single biostrings sequence object
# inPWM: list of PWMs
calcPWMMatches <- function(inSeq, inPWM){
	outScores <- lapply(inPWM, function(x){
		matchPWM(x, inSeq[[1]])
	})
	names(outScores) <- names(inPWM)
	outRanges <- lapply(outScores, function(x){
		x@ranges
	})
	return(list(fullInfo=outScores, pwmIntervals=outRanges))
}

rn5_pwmMatch <- calcPWMMatches(rn5_utrSeq, pwmDat)
hg19_pwmMatch <- calcPWMMatches(hg19_utrSeq, pwmDat)
mm10_pwmMatch <- calcPWMMatches(mm10_utrSeq, pwmDat)


## @knitr extractBlatResults
pslHeader <- c("match", "mismatch", "repMatch", "Ns", "QGapCount", "QGapBases", "TGapCount", "TGapBases", "strand", "QName", "QSize", "QStart", "QEnd", "TName", "TSize", "TStart", "TEnd", "blockCount", "blockSizes", "qStarts", "tStarts")

rn5_mm10 <- read.table("rn5_mm10_blat.psl", sep="\t", header=F)
rn5_hg19 <- read.table("rn5_hg19_blat.psl", sep="\t", header=F)
mm10_hg19 <- read.table("mm10_hg19_blat.psl", sep="\t", header=F)

names(rn5_mm10) <- names(rn5_hg19) <- names(mm10_hg19) <- pslHeader


## @knitr getBLATIntervals
# for working with matches on the rat utr sequence
rn5_mm10_locs <- as.numeric(strsplit(rn5_mm10[1, "qStarts"], ",")[[1]])
rn5_mm10_sizes <- as.numeric(strsplit(rn5_mm10[1, "blockSizes"], ",")[[1]])
rn5_mm10_intervals <- GRanges(seqnames=Rle("chr18"),
															ranges=IRanges(start=rn5_mm10_locs, width=rn5_mm10_sizes),
															strand="+")
rn5_mm10_intervals <- shift(rn5_mm10_intervals, start(rn5_utrLoc))
names(rn5_mm10_intervals) <- as.character(seq(1, length(rn5_mm10_intervals)))

rn5_hg19_locs <- as.numeric(strsplit(rn5_hg19[1, "qStarts"], ",")[[1]])
rn5_hg19_sizes <- as.numeric(strsplit(rn5_hg19[1, "blockSizes"], ",")[[1]])
rn5_hg19_intervals <- GRanges(seqnames=Rle("chr18"),
															ranges=IRanges(start=rn5_hg19_locs, width=rn5_hg19_sizes),
															strand="+")
rn5_hg19_intervals <- shift(rn5_hg19_intervals, start(rn5_utrLoc))
names(rn5_hg19_intervals) <- as.character(seq(1, length(rn5_hg19_intervals)))

# for working with the 
hg19_rn5_locs <- as.numeric(strsplit(rn5_hg19[1, "tStarts"], ",")[[1]])
hg19_rn5_intervals <- GRanges(seqnames=seqnames(hg19_utrLoc), 
															ranges=IRanges(start=hg19_rn5_locs, width=rn5_hg19_sizes),
															strand="+")
names(hg19_rn5_intervals) <- names(rn5_hg19_intervals)

mm10_hg19_locs <- as.numeric(strsplit(mm10_hg19[1, "qStarts"], ",")[[1]])
mm10_hg19_sizes <- as.numeric(strsplit(mm10_hg19[1, "blockSizes"], ",")[[1]])
mm10_hg19_intervals <- GRanges(seqnames=Rle("chr18"), 
															 ranges=IRanges(start=mm10_hg19_locs, width=mm10_hg19_sizes),
															 strand="+")
mm10_hg19_intervals <- shift(mm10_hg19_intervals, start(mm10_utrLoc))

mm10_rn5_locs <- as.numeric(strsplit(rn5_mm10[1, "tStarts"], ",")[[1]])
mm10_rn5_intervals <- GRanges(seqnames=Rle("chr18"),
															ranges=IRanges(start=mm10_rn5_locs, width=rn5_mm10_sizes),
															strand="+")
names(mm10_rn5_intervals) <- names(rn5_mm10_intervals)

hg19_mm10_locs <- as.numeric(strsplit(mm10_hg19[1, "tStarts"], ",")[[1]])
hg19_mm10_intervals <- GRanges(seqnames=Rle("chr5"),
															 ranges=IRanges(start=hg19_mm10_locs, width=mm10_hg19_sizes),
															 strand="+")
names(hg19_mm10_intervals) <- names(mm10_hg19_intervals) <- seq(1, length(mm10_hg19_intervals))


nR2H <- length(rn5_hg19_intervals)
nR2M <- length(rn5_mm10_intervals)


# note: because of what we have done, the PWM intervals for both rat, mouse and human will need to be shifted before examining overlaps


## @knitr clustalWAlignmentIntervals
seqAlign <- read.DNAStringSet("clustalw_alignment.fa", "fasta", use.names=T)
seqAlign <- t(as.matrix(seqAlign))
colnames(seqAlign) <- c("rn5", "mm10", "hg19")

genCoord <- function(inSeq){
	inSeq <- paste(inSeq, sep="", collapse="")
	dnaLoc <- gregexpr("[A-Z]+", inSeq)[[1]]
	seqLens <- attr(dnaLoc, "match.length")
	startCoord <- 1 # this gets updated after every iteration
	
	outCoord <- rep(0, nchar(inSeq))
	
	invisible(sapply(seq(1, length(dnaLoc)), function(whichMatch){
		changeIndx <- c(dnaLoc[whichMatch], dnaLoc[whichMatch] + seqLens[whichMatch]-1)
		endCoord <- startCoord + seqLens[whichMatch] - 1
		outCoord[seq(changeIndx[1], changeIndx[2])] <<- seq(startCoord, endCoord)
		startCoord <<- endCoord + 1
	}))
	return(outCoord)
}

seqCoord <- apply(seqAlign, 2, genCoord)

hasGap <- (seqAlign[,1] == "-") | (seqAlign[,2] == "-") | (seqAlign[,3] == "-")

match2 <- (seqAlign[,1] == seqAlign[,3]) | (seqAlign[,2] == seqAlign[,3]) # rat matches human or mouse matches human
match2 <- match2 & !hasGap
rleMatch <- Rle(match2)
rleSummary <- cbind(runValue(rleMatch), runLength(rleMatch), start(rleMatch), end(rleMatch))
colnames(rleSummary) <- c("value", "length", "start", "end")

rleTrue <- rleSummary[,1] == TRUE
# hist(rleSummary[rleTrue,2], breaks=40, ylim=c(0,20))

getIntervals <- function(inCoord, rleSummary, minLength=8){
	isTrue <- rleSummary[, "value"] == TRUE
	hasMin <- rleSummary[, "length"] >= minLength
	rleSummary <- rleSummary[(isTrue & hasMin),]
	
	IRanges(start=inCoord[rleSummary[, "start"]], end=inCoord[rleSummary[, "end"]])
}

rn5_clustal_intervals <- getIntervals(seqCoord[,"rn5"], rleSummary)
rn5_cwInt <- GRanges(seqnames=as.character(seqnames(rn5_utrLoc)),
																			ranges=shift(rn5_clustal_intervals, start(rn5_utrLoc)),
																			strand=as.character(strand(rn5_utrLoc)))
names(rn5_cwInt) <- as.character(seq(1, length(rn5_cwInt)))

hg19_clustal_intervals <- getIntervals(seqCoord[,"hg19"], rleSummary)
hg19_cwInt <- GRanges(seqnames=as.character(seqnames(hg19_utrLoc)),
																			ranges=shift(hg19_clustal_intervals, start(hg19_utrLoc)),
																			strand=as.character(strand(hg19_utrLoc)))
mm10_clustal_intervals <- getIntervals(seqCoord[,"mm10"], rleSummary)
mm10_cwInt <- GRanges(seqnames=as.character(seqnames(mm10_utrLoc)),
																			ranges=shift(mm10_clustal_intervals, start(mm10_utrLoc)),
																			strand=as.character(strand(mm10_utrLoc)))

names(mm10_cwInt) <- names(hg19_cwInt) <- names(rn5_cwInt)


## @knitr countRatPWMs
rn5_pwmCount <- sapply(rn5_pwmMatch$pwmIntervals, length)

hasMatch <- rn5_pwmCount > 2

ratPWMTable <- data.frame(pwmID=names(rn5_pwmCount)[hasMatch], pwmCount=rn5_pwmCount[hasMatch])
names(protID) <- pwmID[,1]
ratPWMTable$protID <- protID[ratPWMTable$pwmID]
ratPWMTable$geneSymbol <- pwmInfo[ratPWMTable$protID, "geneSymbol"]
names(ratPWMTable) <- c("RBPDB PWM ID", "PWM Count", "RBPDB Protein ID", "Gene Symbol")
ratPWMTable <- ratPWMTable[, c(4, 2, 1, 3)]
ratPWMTable <- ratPWMTable[order(ratPWMTable[,"PWM Count"], decreasing=T),]
write.table(ratPWMTable, file="ratPWMCounts.txt", sep="\t", row.names=F, col.names=T)

print(xtable(ratPWMTable), type = 'html', html.table.attributes = 'style="border-spacing:20px 5px;"', include.rownames=FALSE, sanitize.text.function = force)


## @knitr outputRanges
getShiftedRanges <- function(inRanges, inUTR){
	outRanges <- GRanges(Rle(as.character(seqnames(inUTR))), ranges=shift(inRanges, start(inUTR)), 
											 strand=Rle(as.character(strand(inUTR))))
	return(outRanges)
}

usePWMs <- ratPWMTable[(ratPWMTable[, "Gene Symbol"] %in% c("FUS", "ELAVL1")), "RBPDB PWM ID"]
names2PWM <- ratPWMTable[usePWMs, "Gene Symbol"]
names(names2PWM) <- rownames(ratPWMTable[usePWMs,])
rn5_pwmIntervals <- rn5_pwmMatch$pwmIntervals
rn5_pwmRanges <- lapply(rn5_pwmIntervals[usePWMs], getShiftedRanges, rn5_utrLoc)

hg19_pwmIntervals <- hg19_pwmMatch$pwmIntervals
hg19_pwmRanges <- lapply(hg19_pwmIntervals[usePWMs], getShiftedRanges, hg19_utrLoc)

mm10_pwmIntervals <- mm10_pwmMatch$pwmIntervals
mm10_pwmRanges <- lapply(mm10_pwmIntervals[usePWMs], getShiftedRanges, mm10_utrLoc)


## @knitr outputTracks
exportGRanges <- function(inRange, fileCon, useName="nothing", useColor=as.integer(c(0,0,0))){
	newTrack <- as(new("TrackLine", name=useName, color=useColor), "character")
	cat(newTrack, file=fileCon, sep="\n")
	export(object=inRange, con=fileCon, format="bed")
}


fileCon <- file("rn5_data.bed", open="w+")
exportGRanges(rn5_mm10_intervals, fileCon, useName="Blat with Mm10", useColor=as.integer(c(0,0,0)))
exportGRanges(rn5_hg19_intervals, fileCon, useName="Blat with Hg19", useColor=as.integer(c(0,0,0)))
exportGRanges(rn5_cwInt, fileCon, useName="Clustal Omega three way alignment", useColor=as.integer(c(0,0,0)))

invisible(lapply(names(rn5_pwmRanges), function(x){
	tmpMatch <- rn5_pwmRanges[[x]]
	useName <- names2PWM[x]
	useColor <- as.integer(c(0,0,255))
	exportGRanges(tmpMatch, fileCon, useName, useColor)
}))

close(fileCon)

fileCon <- file("hg19_data.bed", open="w+")
exportGRanges(hg19_rn5_intervals, fileCon, useName="Blat with Rn5", useColor=as.integer(c(0,0,0)))
exportGRanges(hg19_mm10_intervals, fileCon, useName="Blat with Mm10", useColor=as.integer(c(0,0,0)))
exportGRanges(hg19_cwInt, fileCon, useName="Clustal Omega three way alignment", useColor=as.integer(c(0,0,0)))
invisible(lapply(names(hg19_pwmRanges), function(x){
	tmpMatch <- hg19_pwmRanges[[x]]
	useName <- names2PWM[x]
	useColor <- as.integer(c(0,0,255))
	exportGRanges(tmpMatch, fileCon, useName, useColor)
}))

close(fileCon)

fileCon <- file("mm10_data.bed", open="w+")
exportGRanges(mm10_rn5_intervals, fileCon, useName="Blat with Rn5", useColor=as.integer(c(0,0,0)))
exportGRanges(mm10_hg19_intervals, fileCon, useName="Blat with Hg19", useColor=as.integer(c(0,0,0)))
exportGRanges(mm10_cwInt, fileCon, useName="Clustal Omega three way alignment", useColor=as.integer(c(0,0,0)))
invisible(lapply(names(mm10_pwmRanges), function(x){
	tmpMatch <- mm10_pwmRanges[[x]]
	useName <- names2PWM[x]
	useColor <- as.integer(c(0,0,255))
	exportGRanges(tmpMatch, fileCon, useName, useColor)
}))

close(fileCon)


## @knitr allGeneData
geneData <- read.table("hg19_ensGene_081112.txt.gz", stringsAsFactors=FALSE, header=F)
names(geneData) <- c("bin", "transID", "chr", "strand", "txStart", "txEnd", "cdsStart", "cdsEnd", "exonCount",
										 "exonStarts", "exonEnds", "score", "name2", "cdsStartStat", "cdsEndStat", "exonFrames")
utr1 <- GRanges(seqnames=Rle(geneData$chr), strand=Rle(geneData$strand), ranges=IRanges(geneData$txStart, geneData$cdsStart), transID=geneData$transID, geneID=geneData$name2) # from tx start to cds start

utr2 <- GRanges(seqnames=Rle(geneData$chr), strand=Rle(geneData$strand), ranges=IRanges(geneData$cdsEnd, geneData$txEnd), transID=geneData$transID, geneID=geneData$name2) # from cds end to tx end

names(utr1) <- names(utr2) <- geneData$transID

useUTR <- c(utr1[strand(utr1) == "-"], utr2[strand(utr2) == "+"])

useUTR <- useUTR[sample(length(useUTR), 2000)]


## @knitr fusOverlaps
fusBinding <- import("hg19_allChr_mergedFus.bed.gz", "bed", asRangedData=FALSE)
strand(fusBinding) <- Rle(elementMetadata(fusBinding)$name)

fus_hg19_cwIntersect <- intersect(hg19_cwInt, fusBinding)
newTrack <- as(new("TrackLine", name="Fus Binding and Homology", color=as.integer(c(0,0,0))), "character")
cat(newTrack, file="hg19_cw_fusIntersect.bed", sep="\n")
export(fus_hg19_cwIntersect, "hg19_cw_fusIntersect.bed", "bed", append=T)


## @knitr fusCoverage
hg19_utr_fus <- intersect(fusBinding, hg19_utrLoc)
hg19_utr_cov_fus <- sum(width(hg19_utr_fus)) / width(hg19_utrLoc)

useUTR2 <- subsetByOverlaps(useUTR, fusBinding, type="any")
fusZero <- rep(0, sum(!(names(useUTR) %in% names(useUTR2))))
fusBinding2 <- subsetByOverlaps(fusBinding, useUTR2, type="any")

covFunction <- function(utrIndx){
	tmpUTR <- useUTR2[utrIndx]
	fusInt <- intersect(tmpUTR, fusBinding2[utr2Fus2[[utrIndx]]])
	sum(width(fusInt)) / width(tmpUTR)
}

utr2Fus <- as.matrix(findOverlaps(useUTR2, fusBinding2, type="any"))

utr2Fus2 <- split(utr2Fus[,2], utr2Fus[,1])

require(snowfall)
sfInit(cpus=6, parallel=T)
sfLibrary(GenomicRanges)
sfExport("covFunction", "useUTR2", "fusBinding2", "utr2Fus2")
coverage_utrFus <- sfSapply(seq(1, length(utr2Fus2)), covFunction)
sfStop()
coverage_utrFus <- c(coverage_utrFus, fusZero)
save(coverage_utrFus, hg19_utr_cov_fus, file="fus_coverage.RData")


## @knitr elavl1Overlaps
elavData <- read.table("hg19_elavl1_081112.txt.gz", stringsAsFactors=F, header=F, sep="\t")
names(elavData) <- c("bin", "chrom", "chromStart", "chromEnd", "name", "score", "strand", "signalValue",
										 "pValue", "qValue")

elavBinding <- GRanges(seqnames=elavData$chrom, 
											 ranges=IRanges(start=elavData$chromStart, end=elavData$chromEnd),
											 score=elavData$score,
											 pval=elavData$pValue,
											 qval=elavData$qValue)

rm(elavData)

tmpElavl1 <- elavBinding
strand(tmpElavl1) <- "+"
newTrack <- as(new("TrackLine", name="Elavl1 RIP-Seq GM12878 Peaks", color=as.integer(c(0,0,0))), "character")
cat(newTrack, file="hg19_elavl1_ripseqPeaks.bed", sep="\n")
export(tmpElavl1, "hg19_elavl1_ripseqPeaks.bed", "bed", append=T)

elav_hg19_cwIntersect <- intersect(hg19_cwInt, elavBinding, ignore.strand=T)
strand(elav_hg19_cwIntersect) <- "+"
newTrack <- as(new("TrackLine", name="Elavl1 Binding and Homology", color=as.integer(c(0,0,0))), "character")
cat(newTrack, file="hg19_cw_elavIntersect.bed", sep="\n")
export(elav_hg19_cwIntersect, "hg19_cw_elavIntersect.bed", "bed", append=T)


## @knitr elavCoverage
hg19_utr_elav <- intersect(elavBinding, hg19_utrLoc, ignore.strand=T)
hg19_utr_elav_cov <- sum(width(hg19_utr_elav)) / width(hg19_utrLoc)

useUTR2 <- subsetByOverlaps(useUTR, elavBinding, type="any")
elavZero <- rep(0, sum(!(names(useUTR) %in% names(useUTR2))))
elavBinding2 <- subsetByOverlaps(elavBinding, useUTR2, type="any")

covFunction <- function(utrIndx){
	tmpUTR <- useUTR2[utrIndx]
	fusInt <- intersect(tmpUTR, elavBinding2[utr2Elav2[[utrIndx]]], ignore.strand=T)
	sum(width(fusInt)) / width(tmpUTR)
}

utr2Elav <- as.matrix(findOverlaps(useUTR2, elavBinding2, type="any"))

utr2Elav2 <- split(utr2Elav[,2], utr2Elav[,1])

require(snowfall)
sfInit(cpus=6, parallel=T)
sfLibrary(GenomicRanges)
sfExport("covFunction", "useUTR2", "elavBinding2", "utr2Elav2")
coverage_utrElav <- sfSapply(seq(1, length(utr2Elav2)), covFunction)
sfStop()
coverage_utrElav <- c(coverage_utrElav, elavZero)
save(coverage_utrElav, hg19_utr_elav_cov, file="elav_coverage.RData")


## @knitr visCoverage
fusCoverage <- data.frame(Coverage=coverage_utrFus, class="Fus")
elavCoverage <- data.frame(Coverage=coverage_utrElav, class="Elavl1")

allCov <- rbind(fusCoverage, elavCoverage)
camkCov <- data.frame(Coverage=c(hg19_utr_cov_fus, hg19_utr_elav_cov), class=c("Fus", "Elavl1"), y=0)

Cairo(file="coveragePlot.png", type="png", dpi=300)
ggplot() + geom_bar(data=allCov, aes(x=Coverage), binwidth=0.01) + geom_vline(data=camkCov, aes(xintercept=Coverage), colour="red") + facet_grid(. ~ class) + theme_bw() + theme(axis.text = element_text(size=12), axis.title = element_text(size=16), strip.text = element_text(size=20)) + coord_cartesian(ylim=c(0,100))
dev.off()


## @knitr exportCoverage
cat("Fus Coverage All", coverage_utrFus,
		"Fus Camk4", hg19_utr_cov_fus,
		"\n\n", "Elavel1 Coverage All",
		coverage_utrElav,
		"Elavl1 Camk4", hg19_utr_elav_cov, file="coverage.txt", sep="\n")


